<cfcomponent>

	<cfset this.CONFIGURATION_SWITCH_FIELD = "configurationRegistry_configurationSwitch" />

	<cffunction name="init" access="public" output="false" returntype="configurationregistry">
		<cfset variables.my.defaultConfiguration = "" />
		<cfset variables.my.configurations = structNew() />
		<cfreturn this />
	</cffunction>



	<cffunction name="digestRequest" access="public" output="false" returntype="void"
			hint="I MUST be called at the top of every request to process configuration switches">
		<cfif structCount(variables.my.configurations) EQ 0>
			<cfthrow type="IllegalStateException"
					message="This ConfigurationRegistry has not had any configurations registered to it." />
		</cfif>
		<cfif structKeyExists(url, this.CONFIGURATION_SWITCH_FIELD)>
			<cfset session[this.CONFIGURATION_SWITCH_FIELD] = url[this.CONFIGURATION_SWITCH_FIELD] />
		</cfif>
		<cfif structKeyExists(form, this.CONFIGURATION_SWITCH_FIELD)>
			<cfset session[this.CONFIGURATION_SWITCH_FIELD] = form[this.CONFIGURATION_SWITCH_FIELD] />
		</cfif>
		<cfparam name="session.#this.CONFIGURATION_SWITCH_FIELD#" default="#variables.my.defaultConfiguration#" />
	</cffunction>



	<cffunction name="getActiveConfigurationName" access="public" output="false" returntype="string">
		<cfif structKeyExists(session, this.CONFIGURATION_SWITCH_FIELD)
				AND structKeyExists(variables.my.configurations, session[this.CONFIGURATION_SWITCH_FIELD])>
			<cfreturn session[this.CONFIGURATION_SWITCH_FIELD] />
		<cfelse>
			<cfreturn variables.my.defaultConfiguration />
		</cfif>
	</cffunction>



	<cffunction name="getActiveService" access="public" output="false" returntype="service">
		<cfreturn variables.my.configurations[getActiveConfigurationName()] />
	</cffunction>



	<cffunction name="getConfigurationNames" access="public" output="false" returntype="string">
		<cfreturn listSort(structKeyList(variables.my.configurations), "textnocase") />
	</cffunction>



	<cffunction name="register" access="public" output="false" returntype="void">
		<cfargument name="label" type="string" required="true" />
		<cfargument name="service" type="service" required="true" />
		<cfif structKeyExists(variables.my.configurations, label)>
			<cfthrow type="IllegalArgumentException"
					message="Configuration labels must be unique and the label '#label#' is already registered" />
		</cfif>
		<cfset variables.my.configurations[label] = service />
		<cfif variables.my.defaultConfiguration EQ "">
			<cfset setDefault(label) />
		</cfif>
	</cffunction>



	<cffunction name="setDefault" access="public" output="false" returntype="void">
		<cfargument name="label" type="string" required="true" />
		<cfset variables.my.defaultConfiguration = label />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>