<cfcomponent>

	<cfset this.SCHEMA_VERSION = 2 />
	<cfset this.SCHEMA_VERSION_KEY = "schema_version" />

	<cffunction name="init" access="public" output="false" returntype="database">
		<cfargument name="dsn" type="string" required="true" />
		<cfargument name="tablePrefix" type="string" required="true" />
		<cfset variables.my.dsn = dsn />
		<cfset variables.prefix = tablePrefix />
		<cfreturn this />
	</cffunction>



	<cffunction name="upgradeIfNeeded" access="public" output="false" returntype="void">
		<cfset var get = "" />
		<cfset var version = "" />

		<cfset ensureTableExists("schema_info") />
		<cfset ensureTableExists("location") />
		<cfset ensureTableExists("product_category") />
		<cfset ensureTableExists("product") />
		<cfset ensureTableExists("purchase") />
		<cfset ensureTableExists("line_item") />

		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT value
			FROM #prefix#schema_info
			WHERE id = <cfqueryparam cfsqltype="cf_sql_varchar" value="#this.SCHEMA_VERSION_KEY#" />
		</cfquery>
		<cfloop from="#get.value + 1#" to="#this.SCHEMA_VERSION#" index="version">
			<cfinvoke method="updateToVersion_#version#" />
		</cfloop>
	</cffunction>



	<cffunction name="createTable_line_item" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#line_item` (
			  `id` int(10) unsigned NOT NULL auto_increment,
			  `purchaseId` int(10) unsigned NOT NULL default '0',
			  `productId` int(10) unsigned NOT NULL default '0',
			  `quantity` decimal(7,2) NOT NULL default '0.00',
			  `price` decimal(7,2) NOT NULL default '0.00',
			  PRIMARY KEY  (`id`),
			  UNIQUE KEY `uk_productId_purchaseId_price` (`productId`,`purchaseId`,`price`),
			  KEY `k_purchaseId` (`purchaseId`),
			  FOREIGN KEY (`productId`) REFERENCES `#prefix#product` (`id`),
			  FOREIGN KEY (`purchaseId`) REFERENCES `#prefix#purchase` (`id`)
			) ENGINE=InnoDB
		</cfquery>
	</cffunction>



	<cffunction name="createTable_location" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#location` (
			  `id` int(10) unsigned NOT NULL auto_increment,
			  `createDate` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
			  `name` varchar(255) NOT NULL default '',
			  PRIMARY KEY  (`id`),
			  UNIQUE KEY `name` (`name`)
			) ENGINE=InnoDB
		</cfquery>
	</cffunction>



	<cffunction name="createTable_product" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#product` (
			  `id` int(10) unsigned NOT NULL auto_increment,
			  `categoryId` int(10) unsigned default NULL,
			  `createDate` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
			  `name` varchar(255) NOT NULL default '',
			  PRIMARY KEY  (`id`),
			  UNIQUE KEY `name` (`name`),
			  KEY `k_categoryId` (`categoryId`),
			  FOREIGN KEY (`categoryId`) REFERENCES `#prefix#product_category` (`id`)
			) ENGINE=InnoDB
		</cfquery>
	</cffunction>



	<cffunction name="createTable_product_category" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#product_category` (
			  `id` int(10) unsigned NOT NULL auto_increment,
			  `createDate` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
			  `name` varchar(255) NOT NULL default '',
			  PRIMARY KEY  (`id`),
			  UNIQUE KEY `uk_name` (`name`)
			) ENGINE=InnoDB
		</cfquery>
	</cffunction>



	<cffunction name="createTable_purchase" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#purchase` (
			  `id` int(10) unsigned NOT NULL auto_increment,
			  `locationId` int(10) unsigned NOT NULL default '0',
			  `date` date NOT NULL default '0000-00-00',
			  PRIMARY KEY  (`id`),
			  KEY `k_locationId` (`locationId`),
			  FOREIGN KEY (`locationId`) REFERENCES `#prefix#location` (`id`)
			) ENGINE=InnoDB
		</cfquery>
	</cffunction>



	<cffunction name="createTable_schema_info" access="private" output="false" returntype="void">
		<cfquery datasource="#variables.my.dsn#" name="">
			CREATE TABLE `#prefix#schema_info` (
			  `id` varchar(255) NOT NULL default '',
			  `value` text NOT NULL,
			  PRIMARY KEY  (`id`)
			) ENGINE=InnoDB
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="">
			INSERT INTO #prefix#schema_info
				(id, value
				)
			VALUES
				(<cfqueryparam cfsqltype="cf_sql_varchar" value="#this.SCHEMA_VERSION_KEY#" />,
				 <cfqueryparam cfsqltype="cf_sql_integer" value="#this.SCHEMA_VERSION#" />
				)
		</cfquery>
	</cffunction>



	<cffunction name="ensureTableExists" access="private" output="false" returntype="void">
		<cfargument name="name" type="string" required="true" />
		<cftry>
			<cfquery datasource="#variables.my.dsn#" name="get">
				SELECT COUNT(id)
				FROM #prefix##name#
			</cfquery>
			<cfcatch type="database">
				<cfinvoke method="createTable_#name#" />
			</cfcatch>
		</cftry>
	</cffunction>



	<cffunction name="updateToVersion_2" access="private" output="false" returntype="void">
		<!--- no-op upgrade --->
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#schema_info SET
				value = 2
			WHERE id = <cfqueryparam cfsqltype="cf_sql_varchar" value="#this.SCHEMA_VERSION_KEY#" />
		</cfquery>
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>