<cfcomponent output="false">

	<cffunction name="init" access="public" output="false" returntype="service">
		<cfargument name="dsn" type="string" required="true" />
		<cfargument name="tablePrefix" type="string" default="" />
		<cfargument name="bundleName" type="string" default="purchase" />
		<cfset variables.my.dsn = dsn />
		<cfif len(trim(tablePrefix)) EQ 0>
			<cfset variables.prefix = "" />
		<cfelse>
			<cfset variables.prefix = REReplace(tablePrefix, "(^ +)|([ _]+$)", "", "all") & "_" />
		</cfif>
		<cfset createObject("component", "database").init(variables.my.dsn, variables.prefix).upgradeIfNeeded() />
		<cfset variables.my.bundle = createObject("component", "resourcebundle").init(bundleName) />
		<cfreturn this />
	</cffunction>



	<cffunction name="ensureProductExists" access="private" output="false" returntype="numeric">
		<cfargument name="product" type="string" required="true" />
		<cfset var get = "" />
		<cfset product = trim(product) />
		<cfif len(product) EQ 0>
			<cfthrow type="IllegalArgumentException" />
		</cfif>
		<cfquery datasource="#variables.my.dsn#" name="">
			INSERT IGNORE INTO #prefix#product
				(name
				)
			VALUES
				(<cfqueryparam cfsqltype="cf_sql_varchar" value="#product#" />
				)
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT id
			FROM #prefix#product product
			WHERE name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#product#" />
		</cfquery>
		<cfreturn get.id />
	</cffunction>



	<cffunction name="ensureLocationExists" access="private" output="false" returntype="numeric">
		<cfargument name="location" type="string" required="true" />
		<cfset var get = "" />
		<cfset location = trim(location) />
		<cfif len(location) EQ 0>
			<cfthrow type="IllegalArgumentException" />
		</cfif>
		<cfquery datasource="#variables.my.dsn#" name="">
			INSERT IGNORE INTO #prefix#location
				(name
				)
			VALUES
				(<cfqueryparam cfsqltype="cf_sql_varchar" value="#location#" />
				)
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT id
			FROM #prefix#location location
			WHERE name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#location#" />
		</cfquery>
		<cfreturn get.id />
	</cffunction>



	<cffunction name="getCategoryList" access="public" output="false" returntype="query">
		<cfset var getCategoryList = "" />
		<cfquery datasource="#variables.my.dsn#" name="getCategoryList">
			SELECT category.id, category.createDate, category.name,
				(
					SELECT COUNT(*)
					FROM #prefix#product
					WHERE categoryId = category.id
				) AS productCount,
				(
					SELECT COUNT(*)
					FROM #prefix#product product
						INNER JOIN #prefix#line_item lineitem ON product.id = lineitem.productId
					WHERE product.categoryId = category.id
				) AS purchaseCount,
				(
					SELECT SUM(price * quantity)
					FROM #prefix#product product
						INNER JOIN #prefix#line_item lineitem ON product.id = lineitem.productId
					WHERE product.categoryId = category.id
				) AS purchaseTotal
			FROM #prefix#product_category category
			ORDER BY name
		</cfquery>
		<cfreturn getCategoryList />
	</cffunction>



	<cffunction name="getLastProductPrice" access="public" output="false" returntype="string">
		<cfargument name="name" type="string" required="true" />
		<cfset var getLastProductPrice = "" />
		<cfquery datasource="#variables.my.dsn#" name="getLastProductPrice">
			SELECT lineitem.price
			FROM #prefix#line_item lineitem
				INNER JOIN #prefix#product product ON lineitem.productId = product.id
				INNER JOIN #prefix#purchase purchase ON lineitem.purchaseId = purchase.id
			WHERE product.name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#" />
			ORDER BY purchase.date DESC, purchase.id DESC
			LIMIT 1
		</cfquery>
		<cfreturn getLastProductPrice.price />
	</cffunction>



	<cffunction name="getLineItemsForPurchase" access="public" output="false" returntype="query">
		<cfargument name="purchaseId" type="numeric" required="true" />
		<cfset var getLineItemsForPurchase = "" />
		<cfquery datasource="#variables.my.dsn#" name="getLineItemsForPurchase">
			SELECT lineitem.quantity, lineitem.price,
				product.id AS productId, product.name AS productName,
				category.id AS categoryId, category.name AS category
			FROM #prefix#line_item lineitem
				INNER JOIN #prefix#product product ON lineitem.productId = product.id
				LEFT OUTER JOIN #prefix#product_category category ON product.categoryId = category.id
			WHERE lineitem.purchaseId = <cfqueryparam cfsqltype="cf_sql_integer" value="#purchaseId#" />
			ORDER BY category.name, category.id,
				product.name, product.id
		</cfquery>
		<cfreturn getLineItemsForPurchase />
	</cffunction>



	<cffunction name="getLocationList" access="public" output="false" returntype="query">
		<cfset var getLocationList = "" />
		<cfquery datasource="#variables.my.dsn#" name="getLocationList">
			SELECT location.id, location.createDate, location.name,
				(
					SELECT COUNT(*)
					FROM #prefix#purchase purchase
					WHERE purchase.locationId = location.id
				) AS purchaseCount,
				(
					SELECT SUM(price * quantity)
					FROM #prefix#purchase purchase
						INNER JOIN #prefix#line_item lineitem ON purchase.id = lineitem.purchaseId
					WHERE purchase.locationId = location.id
				) AS purchaseTotal
			FROM #prefix#location location
			ORDER BY name
		</cfquery>
		<cfreturn getLocationList />
	</cffunction>



	<cffunction name="getLocationNamesStartingWith" access="public" output="false" returntype="query">
		<cfargument name="name" type="string" required="true" />
		<cfargument name="limit" type="numeric" default="15" />
		<cfset var getLocationNamesStartingWith = "" />
		<cfquery datasource="#variables.my.dsn#" name="getLocationNamesStartingWith">
			SELECT name
			FROM #prefix#location location
			WHERE name LIKE <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#%" />
			ORDER BY name
			LIMIT <cfqueryparam cfsqltype="cf_sql_integer" value="#limit#" />
		</cfquery>
		<cfreturn getLocationNamesStartingWith />
	</cffunction>



	<cffunction name="getProductIndex" access="public" output="false" returntype="query">
		<cfset var getProductIndex = "" />
		<cfquery datasource="#variables.my.dsn#" name="getProductIndex">
				SELECT UPPER(LEFT(name, 1)) AS initial, COUNT(*) AS productCount
				FROM #prefix#product product
				GROUP BY initial
			UNION
				SELECT "-", (
						SELECT COUNT(*)
						FROM #prefix#product product
						WHERE categoryId IS NULL
					) AS productCount
		</cfquery>
		<cfreturn getProductIndex />
	</cffunction>



	<cffunction name="getProductList" access="public" output="false" returntype="query">
		<cfargument name="startsWith" type="string" required="false" />
		<cfargument name="categoryId" type="numeric" required="false" />
		<cfset var getProductList = "" />
		<cfquery datasource="#variables.my.dsn#" name="getProductList">
			SELECT product.id, product.createDate, product.name,
				category.id AS categoryId, category.name AS category,
				(
					SELECT SUM(price * quantity) / SUM(quantity)
					FROM #prefix#line_item
					WHERE productId = product.id
				) AS averagePrice,
				(
					SELECT SUM(quantity)
					FROM #prefix#line_item
					WHERE productId = product.id
				) AS purchaseCount,
				(
					SELECT SUM(price * quantity)
					FROM #prefix#line_item
					WHERE productId = product.id
				) AS total
			FROM #prefix#product product
				LEFT OUTER JOIN #prefix#product_category category ON product.categoryID = category.id
			WHERE 1 = 1
			<cfif structKeyExists(arguments, "startsWith")>
				<cfif startsWith EQ "Uncategorized">
					AND product.categoryID IS NULL
				<cfelse>
					AND product.name LIKE <cfqueryparam cfsqltype="cf_sql_varchar" value="#startsWith#%" />
				</cfif>
			</cfif>
			<cfif structKeyExists(arguments, "categoryId")>
				AND product.categoryID = <cfqueryparam cfsqltype="cf_sql_integer" value="#categoryId#" />
			</cfif>
			ORDER BY IF(category.id IS NULL, 0, 1),
				category.name,
				product.name
		</cfquery>
		<cfreturn getProductList />
	</cffunction>



	<cffunction name="getProductNamesStartingWith" access="public" output="false" returntype="query">
		<cfargument name="name" type="string" required="true" />
		<cfargument name="limit" type="numeric" default="15" />
		<cfset var getProductNamesStartingWith = "" />
		<cfquery datasource="#variables.my.dsn#" name="getProductNamesStartingWith">
			SELECT name
			FROM #prefix#product product
			WHERE name LIKE <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#%" />
			ORDER BY name
			LIMIT <cfqueryparam cfsqltype="cf_sql_integer" value="#limit#" />
		</cfquery>
		<cfreturn getProductNamesStartingWith />
	</cffunction>



	<cffunction name="getPurchaseDetail" access="public" output="false" returntype="query">
		<cfargument name="purchaseId" type="numeric" required="true" />
		<cfreturn getPurchaseList(purchaseId = purchaseId) />
	</cffunction>



	<cffunction name="getPurchaseList" access="public" output="false" returntype="query">
		<cfargument name="offset" type="numeric" default="0" />
		<cfargument name="limit" type="numeric" default="10" />
		<cfargument name="startDate" type="numeric" />
		<cfargument name="endDate" type="numeric" />
		<cfargument name="purchaseId" type="numeric" />
		<cfset var getPurchaseList = "" />
		<cfquery datasource="#variables.my.dsn#" name="getPurchaseList">
			SELECT purchase.id, purchase.date,
				location.id AS locationId, location.name AS location,
				(
					SELECT COUNT(*)
					FROM #prefix#line_item
					WHERE purchaseId = purchase.id
				) AS lineItemCount,
				(
					SELECT COUNT(DISTINCT productId)
					FROM #prefix#line_item
					WHERE purchaseId = purchase.id
				) AS productCount,
				(
					SELECT SUM(quantity)
					FROM #prefix#line_item
					WHERE purchaseId = purchase.id
				) AS itemCount,
				(
					SELECT SUM(quantity * price)
					FROM #prefix#line_item
					WHERE purchaseId = purchase.id
				) AS total
			FROM #prefix#purchase purchase
				INNER JOIN #prefix#location location ON purchase.locationId = location.id
			WHERE 1 = 1
				<cfif structKeyExists(arguments, "startDate")>
					AND purchase.date >= <cfqueryparam cfsqltype="cf_sql_timestamp" value="#startDate#" />
				</cfif>
				<cfif structKeyExists(arguments, "endDate")>
					AND purchase.date <= <cfqueryparam cfsqltype="cf_sql_timestamp" value="#endDate#" /> -- >
				</cfif>
				<cfif structKeyExists(arguments, "purchaseId")>
					AND purchase.id = <cfqueryparam cfsqltype="cf_sql_integer" value="#purchaseId#" />
				</cfif>
			ORDER BY date DESC, location, id DESC
			<cfif limit GT 0>
				LIMIT <cfqueryparam cfsqltype="cf_sql_integer" value="#limit#" />
				OFFSET <cfqueryparam cfsqltype="cf_sql_integer" value="#offset#" />
			</cfif>
		</cfquery>
		<cfreturn getPurchaseList />
	</cffunction>



	<cffunction name="getString" access="public" output="false" returntype="string">
		<cfargument name="code" type="string" required="true" />
		<cfreturn variables.my.bundle.getString(code) />
	</cffunction>



	<cffunction name="recordPurchase" access="public" output="false" returntype="numeric">
		<cfargument name="date" type="string" required="true" />
		<cfargument name="location" type="string" required="true" />
		<cfset var locationId = ensureLocationExists(location) />
		<cfset var get = "" />
		<cfif NOT isDate(date)>
			<cfthrow type="IllegalArgumentException" />
		</cfif>
		<cftransaction>
			<cfquery datasource="#variables.my.dsn#" name="">
				INSERT INTO #prefix#purchase
					(date, locationId
					)
				VALUES
					(<cfqueryparam cfsqltype="cf_sql_timestamp" value="#date#" />,
					 <cfqueryparam cfsqltype="cf_sql_integer" value="#locationId#" />
					)
			</cfquery>
			<cfquery datasource="#variables.my.dsn#" name="get">
				SELECT LAST_INSERT_ID() AS id
			</cfquery>
		</cftransaction>
		<cfreturn get.id />
	</cffunction>



	<cffunction name="recordCategory" access="public" output="false" returntype="numeric">
		<cfargument name="name" type="string" required="true" />
		<cfset var get = "" />
		<cftransaction>
			<cfquery datasource="#variables.my.dsn#" name="">
				INSERT INTO #prefix#product_category
					(name
					)
				VALUES
					(<cfqueryparam cfsqltype="cf_sql_varchar" value="#name#" />
					)
			</cfquery>
			<cfquery datasource="#variables.my.dsn#" name="get">
				SELECT LAST_INSERT_ID() AS id
			</cfquery>
		</cftransaction>
		<cfreturn get.id />
	</cffunction>



	<cffunction name="recordLineItem" access="public" output="false" returntype="numeric">
		<cfargument name="purchaseId" type="string" required="true" />
		<cfargument name="product" type="string" required="true" />
		<cfargument name="quantity" type="string" required="true" />
		<cfargument name="price" type="string" required="true" />
		<cfset var productId = ensureProductExists(product) />
		<cfset var get = "" />
		<cfif NOT isNumeric(purchaseId) OR NOT isNumeric(quantity) OR NOT isNumeric(price)>
			<cfthrow type="IllegalArgumentException" />
		</cfif>
		<cftransaction>
			<cfquery datasource="#variables.my.dsn#" name="get">
				SELECT id
				FROM #prefix#line_item lineitem
				WHERE purchaseId = <cfqueryparam cfsqltype="cf_sql_integer" value="#purchaseId#" />
					AND productId = <cfqueryparam cfsqltype="cf_sql_integer" value="#productId#" />
					AND price = ROUND(<cfqueryparam cfsqltype="cf_sql_decimal" value="#price#" /> * 100) / 100
			</cfquery>
			<cfif get.recordCount NEQ 0>
				<cfquery datasource="#variables.my.dsn#" name="">
					UPDATE #prefix#line_item lineitem SET
						quantity = quantity + <cfqueryparam cfsqltype="cf_sql_integer" value="#quantity#" />
					WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#get.id#" />
				</cfquery>
			<cfelse>
				<cfquery datasource="#variables.my.dsn#" name="">
					INSERT INTO #prefix#line_item
						(purchaseId, productId, quantity,
						 price
						)
					VALUES
						(<cfqueryparam cfsqltype="cf_sql_integer" value="#purchaseId#" />,
						 <cfqueryparam cfsqltype="cf_sql_integer" value="#productId#" />,
						 <cfqueryparam cfsqltype="cf_sql_decimal" value="#quantity#" />,
						 <cfqueryparam cfsqltype="cf_sql_decimal" value="#price#" />
						)
				</cfquery>
				<cfquery datasource="#variables.my.dsn#" name="get">
					SELECT LAST_INSERT_ID() AS id
				</cfquery>
			</cfif>
		</cftransaction>
		<cfreturn get.id />
	</cffunction>



	<cffunction name="updateCategoryName" access="public" output="false" returntype="string">
		<cfargument name="categoryId" type="numeric" required="true" />
		<cfargument name="name" type="string" required="true" />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#product_category category SET
				name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#" />
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#categoryId#" />
		</cfquery>
		<cfreturn name />
	</cffunction>



	<cffunction name="updateLocationName" access="public" output="false" returntype="string">
		<cfargument name="locationId" type="numeric" required="true" />
		<cfargument name="name" type="string" required="true" />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#location location SET
				name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#" />
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#locationId#" />
		</cfquery>
		<cfreturn name />
	</cffunction>



	<cffunction name="updateProductCategory" access="public" output="false" returntype="string">
		<cfargument name="productId" type="numeric" required="true" />
		<cfargument name="categoryId" type="numeric" required="true" />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#product product SET
				categoryId = <cfqueryparam cfsqltype="cf_sql_integer" value="#categoryId#" />
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#productId#" />
		</cfquery>
		<cfquery datasource="#variables.my.dsn#" name="get">
			SELECT name
			FROM #prefix#product_category
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#categoryId#" />
		</cfquery>
		<cfreturn get.name />
	</cffunction>



	<cffunction name="updateProductName" access="public" output="false" returntype="string">
		<cfargument name="productId" type="numeric" required="true" />
		<cfargument name="name" type="string" required="true" />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#product product SET
				name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#name#" />
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#productId#" />
		</cfquery>
		<cfreturn name />
	</cffunction>



	<cffunction name="updatePurchaseDate" access="public" output="false" returntype="string">
		<cfargument name="purchaseId" type="numeric" required="true" />
		<cfargument name="date" type="date" required="true" />
		<cfset var get = "" />
		<cfquery datasource="#variables.my.dsn#" name="">
			UPDATE #prefix#purchase purchase SET
				date = <cfqueryparam cfsqltype="cf_sql_timestamp" value="#date#" />
			WHERE id = <cfqueryparam cfsqltype="cf_sql_integer" value="#purchaseId#" />
		</cfquery>
		<cfreturn date />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>